@@ -1,3 +1,5 @@
+import os
+
 from pulp_2to3_migration.app.plugin.api import (
     is_different_relative_url,
     Pulp2to3Importer,
@@ -7,8 +9,6 @@ from pulp_2to3_migration.app.plugin.api import (
 from pulp_file.app.models import FileRemote, FilePublication, FileDistribution
 from pulp_file.app.tasks.publishing import publish
 
-from urllib.parse import urljoin
-
 
 class IsoImporter(Pulp2to3Importer):
     """
@@ -34,7 +34,7 @@ class IsoImporter(Pulp2to3Importer):
         # pulp2 iso importer is compatible only with repos that contain namely PULP_MANIFEST
         url = base_config.get('url')
         if url:
-            base_config['url'] = urljoin(url, 'PULP_MANIFEST')
+            base_config['url'] = os.path.join(url, 'PULP_MANIFEST')
         return FileRemote.objects.update_or_create(name=name, defaults=base_config)
 
 
@@ -63,11 +63,11 @@ class IsoDistributor(Pulp2to3Distributor):
         if not repo_version:
             repo = pulp2distributor.pulp2_repos.filter(not_in_plan=False, is_migrated=True)
             repo_version = repo[0].pulp3_repository_version
-        publication = repo_version.publication_set.first()
+        publication = repo_version.publication_set.filter(complete=True).first()
         if not publication:
             # create publication
             publish('PULP_MANIFEST', repo_version.pk)
-            publication = repo_version.publication_set.first()
+            publication = repo_version.publication_set.filter(complete=True).first()
         # create distribution
         pulp2_config = pulp2distributor.pulp2_config
         base_config = cls.parse_base_config(pulp2distributor, pulp2_config)
